@@ -1,25 +1,17 @@
 /*
- Author: Juan Rada-Vilela, Ph.D.
- Copyright (C) 2010-2014 FuzzyLite Limited
- All rights reserved
+ fuzzylite (R), a fuzzy logic control library in C++.
+ Copyright (C) 2010-2017 FuzzyLite Limited. All rights reserved.
+ Author: Juan Rada-Vilela, Ph.D. <jcrada@fuzzylite.com>
 
  This file is part of fuzzylite.
 
  fuzzylite is free software: you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free
- Software Foundation, either version 3 of the License, or (at your option)
- any later version.
+ the terms of the FuzzyLite License included with the software.
 
- fuzzylite is distributed in the hope that it will be useful, but WITHOUT
- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
- for more details.
-
- You should have received a copy of the GNU Lesser General Public License
- along with fuzzylite.  If not, see <http://www.gnu.org/licenses/>.
-
- fuzzylite™ is a trademark of FuzzyLite Limited.
+ You should have received a copy of the FuzzyLite License along with
+ fuzzylite. If not, see <http://www.fuzzylite.com/license/>.
 
+ fuzzylite is a registered trademark of FuzzyLite Limited.
  */
 
 #include "fl/term/Triangle.h"
@@ -28,38 +20,44 @@ namespace fl {
 
     Triangle::Triangle(const std::string& name, scalar vertexA, scalar vertexB, scalar vertexC, scalar height)
     : Term(name, height), _vertexA(vertexA), _vertexB(vertexB), _vertexC(vertexC) {
-        if (fl::Op::isNaN(vertexC)) {
-            //TODO: Modify FLL to allow passing two parameters only.
-            this->_vertexC = vertexB;
-            this->_vertexB = (vertexA + vertexB) / 2.0;
+        if (Op::isNaN(vertexC)) {
+            this->_vertexC = _vertexB;
+            this->_vertexB = 0.5 * (_vertexA + _vertexB);
         }
     }
 
-    Triangle::~Triangle() {
-    }
+    Triangle::~Triangle() { }
 
     std::string Triangle::className() const {
         return "Triangle";
     }
 
+    Complexity Triangle::complexity() const {
+        return Complexity().comparison(1 + 5).arithmetic(4);
+    }
+
     scalar Triangle::membership(scalar x) const {
-        if (fl::Op::isNaN(x)) return fl::nan;
+        if (Op::isNaN(x)) return fl::nan;
 
         if (Op::isLt(x, _vertexA) or Op::isGt(x, _vertexC))
-            return _height * 0.0;
+            return Term::_height * 0.0;
 
         if (Op::isEq(x, _vertexB))
-            return _height * 1.0;
+            return Term::_height * 1.0;
 
-        if (Op::isLt(x, _vertexB))
-            return _height * (x - _vertexA) / (_vertexB - _vertexA);
-
-        return _height * (_vertexC - x) / (_vertexC - _vertexB);
+        if (Op::isLt(x, _vertexB)) {
+            if (_vertexA == -fl::inf)
+                return Term::_height * 1.0;
+            return Term::_height * (x - _vertexA) / (_vertexB - _vertexA);
+        }
+        if (_vertexC == fl::inf)
+            return Term::_height * 1.0;
+        return Term::_height * (_vertexC - x) / (_vertexC - _vertexB);
     }
 
     std::string Triangle::parameters() const {
         return Op::join(3, " ", _vertexA, _vertexB, _vertexC) +
-                (not Op::isEq(_height, 1.0) ? " " + Op::str(_height) : "");
+                (not Op::isEq(getHeight(), 1.0) ? " " + Op::str(getHeight()) : "");
     }
 
     void Triangle::configure(const std::string& parameters) {
@@ -70,7 +68,7 @@ namespace fl {
             std::ostringstream ex;
             ex << "[configuration error] term <" << className() << ">"
                     << " requires <" << required << "> parameters";
-            throw fl::Exception(ex.str(), FL_AT);
+            throw Exception(ex.str(), FL_AT);
         }
         setVertexA(Op::toScalar(values.at(0)));
         setVertexB(Op::toScalar(values.at(1)));
